/*
    all dimensions are in millimeters
*/

birdhouse();

module birdhouse(explode=0) {
    translate([0, 0, 9-explode])
        rotate([0, 0, 0])
            color("BurlyWood")
                birdhouse_floor();
    
    translate([0, 74+explode, 150])
        rotate([90, 0, 0])
            color("SaddleBrown")
                birdhouse_back();
    
    translate([0, -74-explode, 140])
        rotate([-90, 180, 0]) 
            color("SaddleBrown")
                birdhouse_front();
    
    translate([-74-explode, 0, 150])
        rotate([90, 0, 90])
            color("Chocolate")
                birdhouse_left_side();
    
    translate([74+explode, 0, 150])
        rotate([90, 0, -90])
            color("Chocolate")
                birdhouse_right_side();
   
    translate([0, 83, 300+explode])
        rotate([6.87, 0, 0])
            translate([0, -115, 9])
                color("BurlyWood")
                    birdhouse_roof(); 
}

module birdhouse_floor() {
    cube([144, 144, 18.01], center=true);
}

module birdhouse_back() {
    difference() {
        cube([130, 300, 18], center=true);
        translate([0, -125, 5.5]) {
            cube([130.1, 50.1, 7.1], center=true);
        }
        
        // angle
        translate([0, 150, -9]) {
            rotate([6.87, 0, 0]){
                translate([0, 9, 9]) {
                    cube([130.1, 18, 18.4], center=true);
                }
            }
        }
        
        // holes
        translate([33, -141, 0])
            hole_3mm();
        translate([-33, -141, 0])
            hole_3mm();
    }
}

module birdhouse_front() {
    difference() {
        cube([130, 280, 18], center=true);
        translate([0, -115, 5.5]) {
            cube([130.1, 50.1, 7.1], center=true);
        }
        
        // entry hole
        translate([0, 100, 0]) {
            cylinder(18.1, 15, 15, $fn = 24, center=true);
        }
        
        // steps
        for(v=[-70, -50, -30, -10, 10, 30, 50, 70]) {
            translate([0, v, 7.51]) {
                cube([30, 3, 3], center=true);
            }
        }
        
        // holes
        translate([33, -131, 0])
            hole_3mm();
        translate([-33, -131, 0])
            hole_3mm();
    }
}

module birdhouse_left_side() {
    difference() {
        polyhedron(
            points=[
                [83, -150, 9],
                [-83, -150, 9],
                [-83, 130, 9],
                [83, 150, 9],
                [83, -150, -9],
                [-83, -150, -9],
                [-83, 130, -9],
                [83, 150, -9],
            ],
            faces=[
                [0, 1, 2, 3],
                [4, 7, 6, 5],
                [0, 4, 5, 1],
                [7, 3, 2, 6],
                [4, 0, 3, 7],
                [1, 5, 6, 2],
            ]
        );
        translate([0, -125, 5.5]) {
            cube([144.1, 50.1, 7.1], center=true);
        }
        
        // holes
        translate([33, -141, 0])
            hole_3mm();
        translate([-33, -141, 0])
            hole_3mm();
        
        translate([-74, -70, 0])
            hole_3mm();
        translate([-74, 110, 0])
            hole_3mm();
        
        translate([74, -70, 0])
            hole_3mm();
        translate([74, 70, 0])
            hole_3mm();
    }
}

module birdhouse_right_side() {
    difference() {
        polyhedron(
            points=[
                [83, -150, 9],
                [-83, -150, 9],
                [-83, 150, 9],
                [83, 130, 9],
                [83, -150, -9],
                [-83, -150, -9],
                [-83, 150, -9],
                [83, 130, -9],
            ],
            faces=[
                [0, 1, 2, 3],
                [4, 7, 6, 5],
                [0, 4, 5, 1],
                [7, 3, 2, 6],
                [4, 0, 3, 7],
                [1, 5, 6, 2],
            ]
        );
        translate([0, -125, 5.5]) {
            cube([144.1, 50.1, 7.1], center=true);
        }
        
        // holes
        translate([33, -141, 0])
            hole_3mm();
        translate([-33, -141, 0])
            hole_3mm();
        
        translate([74, -70, 0])
            hole_3mm();
        translate([74, 110, 0])
            hole_3mm();
        
        translate([-74, -70, 0])
            hole_3mm();
        translate([-74, 70, 0])
            hole_3mm();
    }
}

module birdhouse_roof() {
    difference() {
        cube([130+18+18, 230, 18], center=true);
        
        // holes
        translate([74, 65, 0])
                hole_3mm();
        translate([-74, -1, 0])
                hole_3mm();
        translate([74, -1, 0])
                hole_3mm();
        translate([-74, 65, 0])
                hole_3mm();
        
       translate([33, 106, 0])
                hole_3mm();
       translate([-33, 106, 0])
                 hole_3mm();
    }
}



module hole_3mm() {
    cylinder(20.1, 1.5, 1.5, $fn = 24, center=true);
}