/*
    all dimensions are in millimeters
*/

scale();

module scale(explode=0) {
    translate([0, 0, 1-explode]) color("Gray") scale_floor();
    translate([0, 0, 15+explode]) color("Gray") scale_top();
    
    for(v=[
        [20.0, 44.0, 3.5],
        [-20.0, 44.0, 12.5],
        [-20.0, -44.0, 3.5],
        [20.0, -44.0, 12.5],
        ]) translate(v)
            color("Gainsboro")
                spacer();
    
    for(v=[
        [[0, -68.5-explode, 13], [0, 0, 0]],
        [[68.5+explode, 0, 13], [0, 0, 90]],
        [[0, 68.5+explode, 13], [0, 0, 180]],
        [[-68.5-explode, 0, 13], [0, 0, 270]],
        ]) translate(v[0])
            rotate(v[1])
                color("Silver")
                    side();
    
    for(v=[
        [[0, -65-(explode/2), 17], [0, 0, 0]],
        [[65+(explode/2), 0, 17], [0, 0, 90]],
        [[0, 65+(explode/2), 17], [0, 0, 180]],
        [[-65-(explode/2), 0, 17], [0, 0, 270]],
        ]) translate(v[0])
            rotate(v[1])
                color("White")
                    lower_rim();
    
    for(v=[
        [[0, -65-(explode/2), 22], [0, 0, 0]],
        [[65+(explode/2), 0, 22], [0, 0, 90]],
        [[0, 65+(explode/2), 22], [0, 0, 180]],
        [[-65-(explode/2), 0, 22], [0, 0, 270]],
        ]) translate(v[0])
            rotate(v[1])
                color("SlateGray")
                    upper_rim();
}

module scale_floor() {    
    difference() {
        cube([142, 142, 2], center=true);
        
        // 2 mm
        for(v=[
            [68.5, 68.5, 0],
            [68.5, -68.5, 0],
            [-68.5, 68.5, 0],
            [-68.5, -68.5, 0],        
            [-68.5, -25.0, 0],        
            [-68.5, 25.0, 0],        
            [68.5, -25.0, 0],        
            [68.5, 25.0, 0],        
            [-25.0, -68.5, 0],        
            [25.0, -68.5, 0],        
            [-25.0, 68.5, 0],        
            [25.0, 68.5, 0],
        ]) {
            translate(v) {
                rotate([0, 0, 90]) {
                    cylinder(2.1, 1.0, 1.0, $fn = 24, center=true);
                }
            }
        }
        
        // 3 mm
        for(v=[
            [20.0, 47.0, 0],
            [20.0, 41.0, 0],
            [-20.0, -47.0, 0],
            [-20.0, -41.0, 0],
            [15.25, 10.15, 0],
            [-15.25, 10.15, 0],
            [15.25, -10.15, 0],
            [-15.25, -10.15, 0],
            [-50.0, -50.0, 0],
            [-50.0, 50.0, 0],
            [50.0, -50.0, 0],
            [50.0, 50.0, 0],
        ]) {
            translate(v) {
                rotate([0, 0, 90]) {
                    cylinder(2.1, 1.5, 1.5, $fn = 24, center=true);
                }
            }
        }
        
        // 4 mm drains
        for(v=[
            [64.0, 64.0, 0],
            [-64.0, 64.0, 0],
            [64.0, -64.0, 0],
            [-64.0, -64.0, 0],
            [-41.9, 0.0, 0],
            [41.9, 0.0, 0],
        ]) {
            translate(v) {
                rotate([0, 0, 90]) {
                    cylinder(2.1, 2.0, 2.0, $fn = 24, center=true);
                }
            }
        }
     }
}

module scale_top() {  
     difference() {
        cube([134, 134, 2], center=true);
        
        // M2 
        for(v=[
            [64.5, 25.0, 0],
            [64.5, -25.0, 0],
            [-64.5, 25.0, 0],
            [-64.5, -25.0, 0],
            [25.0, 64.5, 0],
            [-25.0, 64.5, 0],
            [25.0, -64.5, 0],
            [-25.0, -64.5, 0],
        ]) {
            translate(v) {
                cylinder(2.1, 1.0, 1.0, $fn = 24, center=true);
            }
        }
        
        // 3 mm
        for(v=[
            [20.0, -47.0, 0],
            [20.0, -41.0, 0],
            [-20.0, 47.0, 0],
            [-20.0, 41.0, 0],
        ]) {
            translate(v) {
                rotate([0, 0, 90]) {
                    cylinder(2.1, 1.5, 1.5, $fn = 24, center=true);
                }
            }
        }
    }
}

module spacer() {
   difference() {
        cube([7, 12, 3], center=true);
        
        // 3 mm
        translate([0, 3, 0]) {
            rotate([0, 0, 90]) {
                cylinder(3.1, 1.5, 1.5, $fn = 24, center=true);
            }
        }
        translate([0, -3, 0]) {
            rotate([0, 0, 90]) {
                cylinder(3.1, 1.5, 1.5, $fn = 24, center=true);
            }
        }
    }
}

module side() {
    difference() {
        cube([142, 5, 22], center=true);
        translate([0, 1.0, 3.5]) {
            cube([142.01, 3.01, 11.01], center=true);
        }
        translate([68.5, 0, 4.5]) {
            cube([5.01, 5.01, 13.01], center=true);
        }
        translate([-68.5, 0, -6.5]) {
            cube([5.01, 5.01, 9.01], center=true);
        }
        
        // M2
        for(v=[
            [25.0, 0.0, -6.5],
            [-25.0, 0.0, -6.5],
            [68.5, 0.0, -6.5]
        ]) {
            translate(v) {
                cylinder(9.1, 1.0, 1.0, $fn = 24, center=true);
            }
        }
        for(v=[
            [25.0, 0.0, 10],
            [-25.0, 0.0, 10],
        ]) {
            translate(v) {
                cylinder(2.1, 1.0, 1.0, $fn = 24, center=true);
            }
        }
        
        // 4mm
        translate([0, 0, -7.0]) {
            rotate([90, 0, 0]) {
                cylinder(5.1, 2.0, 2.0, $fn = 24, center=true);
            }
        }
    }
}

module lower_rim() {
    difference() {
        cube([136, 6, 8], center=true);
        translate([0, 1, 1]) {
            cube([136.1, 6.01, 8.01], center=true);
        }
        
        // 2mm
        for(v=[
            [25.0, 0.5, -3.5],
            [-25.0, 0.5, -3.5],
        ]) {
            translate(v) {
                cylinder(1.1, 1.0, 1.0, $fn = 24, center=true);
            }
        }
        
        // 45 degree cut
        translate([68.0, -3, -5]) {
            rotate([0, 0, 45]) {
                cube([10, 10, 10], center=false);
            }
        }     
        translate([-68.0, -3, -5]) {
            rotate([0, 0, 45]) {
                cube([10, 10, 10], center=false);
            }
        }     
    }
}

module upper_rim() {
    difference() {
        union() {
            translate([0, -3.5, 2.5]) {
                cube([142, 5.0, 1.0], center=true);
            }
            translate([0, -0.5, 0]) {
               cube([142, 1.0, 6.0], center=true);
            }
        }
        
        // 2mm
        for(v=[
            [25.0, -3.5, 2.5],
            [-25.0, -3.5, 2.5],
        ]) {
            translate(v) {
                cylinder(1.1, 1.0, 1.0, $fn = 24, center=true);
            }
        }
    
        // 45 degree cut
        translate([-65, 0, -5]) {
            rotate([0, 0, 135]) {
                cube([10, 10, 10], center=false);
            }
        }
        translate([65, 0, -5]) {
            rotate([0, 0, -45]) {
                cube([10, 10, 10], center=false);
            }
        }     
    }
}    
